#include "frameSwapper.h"

namespace M2Rend{
   void FrameSwapperDefinition::addFrame( const std::string &mainTexture, const std::string & subTexture, int duration /*= -1*/ ){
      std::pair<int, std::pair<std::string, std::string> > newItem;
      newItem.first = duration;
      newItem.second.first = mainTexture;
      newItem.second.second = subTexture;
      frameList.push_back(newItem);
   }

   FrameSwapper::FrameSwapper(){
      defaultDuration = 100;
      previousFrame = 0;
      framesListReference = 0;
   }

   void FrameSwapper::setDefaultFrameDuration( int newDurationMilliseconds ){
      defaultDuration = newDurationMilliseconds;
   }

   void FrameSwapper::start(){
      timer.stop();
      timer.start();
      previousFrame = timer.check();
      frame = framesListReference->frameList.begin();
      nextFrame = getNextFrameTime();
   }

   void FrameSwapper::pause(){
      timer.pause();
   }

   void FrameSwapper::resume(){
      timer.resume();
   }

   bool FrameSwapper::getCurrentFrame( std::string &returnMainTexture, std::string &returnSubTexture ){
      bool hasWrapped = false;
      if(framesListReference->frameList.empty()){
         returnMainTexture = "NULL";
         returnSubTexture = "NULL";
         return false;
      }
      if(nextFrame < timer.check()){
         frame++;
         if(frame == framesListReference->frameList.end()){
            frame = framesListReference->frameList.begin();
            hasWrapped = true;
         }
         nextFrame = getNextFrameTime();
         timer.stop(); timer.start();
      }
      returnMainTexture = frame->second.first;
      returnSubTexture = frame->second.second;
      return hasWrapped;
   }

   int FrameSwapper::getNextFrameTime(){
      int tmpDur;
      tmpDur = frame->first;
      if(tmpDur == -1){
         tmpDur = defaultDuration;
      }
      return tmpDur;
   }

   void FrameSwapper::setFrameList( FrameSwapperDefinition *frameList ){
      framesListReference = frameList;
      frame = framesListReference->frameList.begin();
   }

   FrameSwapperRegister::FrameSwapperRegister(){
      FrameSwapperDefinition tmpSwap;
      tmpSwap.addFrame("NULL", "NULL");
      definitions["NULL"] = tmpSwap;
   }

   void FrameSwapperRegister::newDefinition( const FrameSwapperDefinition &definition, const std::string &name ){
      definitions[name] = definition;
   }

   FrameSwapperDefinition* FrameSwapperRegister::getDefinition( const std::string &name ){
      std::map<std::string, FrameSwapperDefinition>::iterator cell = definitions.find(name);
      if(cell != definitions.end()){
         return &(cell->second);
      }
      return &(definitions["NULL"]);
   }
}
