#ifndef _FRAMESWAPPER_H_
#define _FRAMESWAPPER_H_

#include <list>
#include "Utility/package.h"

namespace M2Rend{
   class FrameSwapperDefinition{
   public:
      void addFrame(const std::string &mainTexture, const std::string & subTexture, int duration = -1);
      void clear(){frameList.clear();}
      std::list<std::pair<int, std::pair<std::string, std::string> > > frameList;
   };

   class FrameSwapper {
   public:
      FrameSwapper();

      void setFrameList(FrameSwapperDefinition *frameList);

      void setDefaultFrameDuration(int newDurationMilliseconds);

      void start();
      void pause();
      void resume();

      //MainTexture and subTexture are return values, not inputs.
      bool getCurrentFrame(std::string &returnMainTexture, std::string &returnSubTexture);
   private:
      int getNextFrameTime();

      FrameSwapperDefinition *framesListReference;

      int defaultDuration;
      M2Util::Stopwatch timer;
      int previousFrame, nextFrame;

      //time for frame, maintexture, subtexture
      std::list<std::pair<int, std::pair<std::string, std::string> > >::iterator frame;
   };

   class FrameSwapperRegister{
   public:
      FrameSwapperRegister();

      void newDefinition(const FrameSwapperDefinition &definition, const std::string &name);
      FrameSwapperDefinition* getDefinition(const std::string &name);
   private:
      std::map<std::string, FrameSwapperDefinition> definitions;
   };
}
#endif
