#include "drawShapes.h"
namespace M2Rend {
   /*************************\
   | ------BoundingBox------ |
   \*************************/

   void BoxAABB::initialize( const Point &startPoint ){
      minPoint = startPoint; maxPoint = startPoint;
   }

   void BoxAABB::initialize( const Point &startPoint, const Point &endPoint){
      initialize(startPoint);
      expandWith(endPoint);
   }

   void BoxAABB::initialize( const BoxAABB &startBox ){
      minPoint = startBox.minPoint; maxPoint = startBox.maxPoint;
   }

   void BoxAABB::expandWith( const Point &comparePoint ){
      minPoint.x = std::min(comparePoint.x, minPoint.x);
      minPoint.y = std::min(comparePoint.y, minPoint.y);
      minPoint.z = std::min(comparePoint.z, minPoint.z);

      maxPoint.x = std::max(comparePoint.x, maxPoint.x);
      maxPoint.y = std::max(comparePoint.y, maxPoint.y);
      maxPoint.z = std::max(comparePoint.z, maxPoint.z);
   }

   void BoxAABB::expandWith( const BoxAABB &compareBox ){
      expandWith(compareBox.minPoint);
      expandWith(compareBox.maxPoint);
   }

   bool BoxAABB::pointContained( double x, double y, double z ){
      if(x >= minPoint.x && y >= minPoint.y && z >= minPoint.z){
         if(x <= maxPoint.x && y <= maxPoint.y && z <= maxPoint.z){
            return true;
         }
      }
      return false;
   }

   bool BoxAABB::pointContained( const Point &comparePoint ){
      return pointContained(comparePoint.x, comparePoint.y, comparePoint.z);
   }

   bool BoxAABB::pointContainedXY( double x, double y ){
      if((x >= minPoint.x && y >= minPoint.y) && (x <= maxPoint.x && y <= maxPoint.y)){
         return true;
      }
      return false;
   }

   bool BoxAABB::pointContainedXY( const Point &comparePoint ){
      return pointContainedXY(comparePoint.x, comparePoint.y);
   }

   /*************************\
   | -------DrawShape------- |
   \*************************/

   void DrawShape::setColor( float R, float G, float B, float A ){
      int elements = (int)Pnt.size();
      for(int i = 0;i < elements;i++){
         Pnt[i].R = R;
         Pnt[i].G = G;
         Pnt[i].B = B;
         Pnt[i].A = A;
      }
   }

   void DrawShape::setColor( Color newColor ){
      int elements = (int)Pnt.size();
      for(int i = 0;i < elements;i++){
         Pnt[i] = newColor;
      }
   }

   void DrawShape::setTexture( GLuint *textureId ){
      texture = textureId;
      hasTexture = true;
   }

   void DrawShape::removeTexture(){
      hasTexture = false;
   }

   double DrawShape::getDepth(){
      if(depthOverride){
         return overrideDepthValue;
      }
      int elements = (int)Pnt.size();
      double total = 0;
      for(int i = 0;i < elements;i++){
         total+=Pnt[i].z;
      }
      total/=(double)elements;
      return total;
   }

   bool DrawShape::operator<( DrawShape &other ){
      return getDepth() < other.getDepth();
   }

   bool DrawShape::operator>( DrawShape &other ){
      return getDepth() > other.getDepth();
   }

   bool DrawShape::operator==( DrawShape &other ){
      return getDepth() == other.getDepth();
   }

   Point DrawShape::getScale(){
      return scaleTo;
   }

   Point DrawShape::getRotation(){
      return rotateTo;
   }

   Point DrawShape::getLocation(){
      return translateTo;
   }

   Point DrawShape::getRelativeLocation(){
      Point resultPoint;
      resultPoint = translateTo;
      if(myParent){
         resultPoint += myParent->getRelativeLocation();
      }
      return resultPoint;
   }

   Point DrawShape::getRelativeScale(){
      Point resultPoint;
      resultPoint = scaleTo;
      if(myParent){
         resultPoint += myParent->getRelativeScale();
      }
      return resultPoint;
   }

   Point DrawShape::getRelativeRotation(){
      Point resultPoint;
      resultPoint = rotateTo;
      if(myParent){
         resultPoint += myParent->getRelativeRotation();
      }
      return resultPoint;
   }

   void DrawShape::scale( double newScale ){
      scale( Point(newScale, newScale, newScale) );
   }

   void DrawShape::scale( double x, double y, double z ){
      scale( Point(x, y, z) );
   }

   void DrawShape::scale( Point scaleValue ){
      scaleTo = scaleValue;
   }

   void DrawShape::incrementScale( double newScale ){
      incrementScale( Point(newScale, newScale, newScale) );
   }

   void DrawShape::incrementScale( double x, double y, double z ){
      incrementScale( Point(x, y, z) );
   }

   void DrawShape::incrementScale( Point scaleValue ){
      scaleTo += scaleValue;
   }

   void DrawShape::placeAt( double x, double y, double z ){
      translateTo.x=x; translateTo.y=y; translateTo.z=z;
   }

   void DrawShape::translate( double x, double y, double z ){
      translateTo.x+= x; translateTo.y+= y; translateTo.z+= z;
   }

   void DrawShape::incrementRotate( double x, double y, double z ){
      rotateTo.x=M2Util::boundBetween(rotateTo.x+=x, 0.0, 360.0);
      rotateTo.y=M2Util::boundBetween(rotateTo.y+=y, 0.0, 360.0);
      rotateTo.z=M2Util::boundBetween(rotateTo.z+=z, 0.0, 360.0);
   }

   void DrawShape::setRotate( double x, double y, double z ){
      rotateTo.x = M2Util::boundBetween(x, 0.0, 360.0);
      rotateTo.y = M2Util::boundBetween(y, 0.0, 360.0);
      rotateTo.z = M2Util::boundBetween(z, 0.0, 360.0);
   }

   void DrawShape::setRotateOrigin( double x, double y, double z ){
      rotateOrigin.x = x; rotateOrigin.y = y; rotateOrigin.z = z;
   }

   void DrawShape::pushMatrix(){
      glPushMatrix();
      if(!translateTo.atOrigin()){
         glTranslated(translateTo.x, translateTo.y, translateTo.z);
      }
      if(!scaleTo.atOrigin()){
         glScaled(scaleTo.x, scaleTo.y, scaleTo.z);
      }
      if(!rotateTo.atOrigin()){
         glTranslated(rotateOrigin.x, rotateOrigin.y, rotateOrigin.z);
         glRotated(rotateTo.x, 1.0, 0.0, 0.0);
         glRotated(rotateTo.y, 0.0, 1.0, 0.0);
         glRotated(rotateTo.z, 0.0, 0.0, 1.0);
         glTranslated(-rotateOrigin.x, -rotateOrigin.y, -rotateOrigin.z);
      }
      if(hasTexture && texture != NULL){
         glEnable(GL_TEXTURE_2D);
         glBindTexture(GL_TEXTURE_2D, *texture);
      }else{
         glDisable(GL_TEXTURE_2D);
      }
   }

   void DrawShape::popMatrix(){
      glPopMatrix();
   }

   void DrawShape::setParent( DrawShape* parentItem ){
      myParent = parentItem;
   }

   BoxAABB DrawShape::getAABB(){
      alertParent("pushMatrix");
      pushMatrix();
      int elements = (int)Pnt.size();
      BoxAABB tmpBox;
      if(elements > 0){
         tmpBox.initialize(renderer->getObjectToWorldPoint(Pnt[0]));
      }
      for(int i = 0;i < elements;i++){
         tmpBox.expandWith(renderer->getObjectToWorldPoint(Pnt[i]));
      }
      popMatrix();
      alertParent("popMatrix");
      return tmpBox;
   }

   PointVolume DrawShape::getWorldPoints(){
      alertParent("pushMatrix");
      pushMatrix();
      int elements = (int)Pnt.size();
      PointVolume pointList;
      for(int i = 0;i < elements;i++){
         pointList.addPoint(renderer->getObjectToWorldPoint(Pnt[i]));
      }
      popMatrix();
      alertParent("popMatrix");
      return pointList;
   }

   Point DrawShape::getWorldPoint(Point local){
      alertParent("pushMatrix");
      pushMatrix();
      Point ourPoint;
      ourPoint = renderer->getObjectToWorldPoint(local);
      popMatrix();
      alertParent("popMatrix");
      return ourPoint;
   }

   /*************************\
   | -------DrawPixel------- |
   \*************************/

   void DrawPixel::setPoint( double x, double y, double z ){
      alertParent("NeedsSort");
      Pnt[0].x = x; Pnt[0].y = y; Pnt[0].z = z;
   }

   void DrawPixel::setPoint( DrawPoint point ){
      alertParent("NeedsSort");
      Pnt[0] = point;
   }

   void DrawPixel::draw(){
      pushMatrix();
      glBegin(GL_POINTS);
      glColor4f(Pnt[0].R, Pnt[0].G, Pnt[0].B, Pnt[0].A);
      glVertex3d(Pnt[0].x, Pnt[0].y, Pnt[0].z);
      glEnd();
      popMatrix();
   }

   /*************************\
   | --------DrawLine------- |
   \*************************/

   void DrawLine::setEnds( double x1, double y1, double z1, double x2, double y2, double z2 ){
      alertParent("NeedsSort");
      Pnt[0].x = std::min(x1, x2); Pnt[0].y = std::min(y1, y2); Pnt[0].z = z1;
      Pnt[1].x = std::max(x2, x1); Pnt[1].y = std::max(y2, y1); Pnt[1].z = z2;
   }

   void DrawLine::setEnds( Point startPoint, Point endPoint ){
      alertParent("NeedsSort");
      Pnt[0] = startPoint;
      Pnt[1] = endPoint;
   }

   void DrawLine::draw(){
      pushMatrix();
      glBegin(GL_LINES);
      for(int i = 0;i < 2;i++){
         glColor4f(Pnt[i].R, Pnt[i].G, Pnt[i].B, Pnt[i].A);
         glVertex3d(Pnt[i].x, Pnt[i].y, Pnt[i].z);
      }
      glEnd();
      popMatrix();
   }

   /*************************\
   | -----DrawRectangle----- |
   \*************************/

   void DrawRectangle::setTwoCorners( double x1, double y1, double z1, double x2, double y2, double z2 ){
      alertParent("NeedsSort");
      Pnt[0].x = std::min(x1,x2); Pnt[0].y = std::min(y1,y2); Pnt[0].z = z1;
      Pnt[1].x = std::min(x1,x2); Pnt[1].y = std::max(y2,y1); Pnt[1].z = (z1+z2)/2;
      Pnt[2].x = std::max(x2,x1); Pnt[2].y = std::max(y2,y1); Pnt[2].z = z2;
      Pnt[3].x = std::max(x2,x1); Pnt[3].y = std::min(y1,y2); Pnt[3].z = (z1+z2)/2;
   }

   void DrawRectangle::setTwoCorners( Point TopLeft, Point BottomRight ){
      alertParent("NeedsSort");
      Pnt[0] = TopLeft;
      Pnt[1].x = TopLeft.x;   Pnt[1].y = BottomRight.y;   Pnt[1].z = (BottomRight.z + TopLeft.z) / 2;
      Pnt[2] = BottomRight;
      Pnt[3].x = BottomRight.x;   Pnt[3].y = TopLeft.y;   Pnt[3].z = (BottomRight.z + TopLeft.z) / 2;
   }

   void DrawRectangle::resetTextureCoordinates(){
      Pnt[0].textureX = 0.0; Pnt[0].textureY = 0.0;
      Pnt[1].textureX = 0.0; Pnt[1].textureY = 1.0;
      Pnt[2].textureX = 1.0; Pnt[2].textureY = 1.0;
      Pnt[3].textureX = 1.0; Pnt[3].textureY = 0.0;
   }

   void DrawRectangle::draw(){
      pushMatrix();
      glBegin(GL_QUADS);
      for(int i = 0;i < 4;i++){
         if(hasTexture){
            glTexCoord2d(Pnt[i].textureX,Pnt[i].textureY);
         }
         glColor4f(Pnt[i].R, Pnt[i].G, Pnt[i].B, Pnt[i].A);
         glVertex3d(Pnt[i].x, Pnt[i].y, Pnt[i].z);
      }
      glEnd();
      popMatrix();
   }

   void AssignTextureToRectangle(DrawRectangle &rectangle, SubTexture *texture, bool flip){
      TexturePoint TmpPoint[4];
      TmpPoint[0].textureX = texture->percentLeftX;
      TmpPoint[0].textureY = texture->percentTopY;

      TmpPoint[1].textureX = texture->percentLeftX+texture->percentWidth;
      TmpPoint[1].textureY = texture->percentTopY;

      TmpPoint[2].textureX = texture->percentLeftX+texture->percentWidth;
      TmpPoint[2].textureY = texture->percentTopY+texture->percentHeight;

      TmpPoint[3].textureX = texture->percentLeftX;
      TmpPoint[3].textureY = texture->percentTopY+texture->percentHeight;

      if(flip){
         std::swap(TmpPoint[0].textureY, TmpPoint[1].textureY);
         std::swap(TmpPoint[0].textureX, TmpPoint[1].textureX);
         std::swap(TmpPoint[2].textureY, TmpPoint[3].textureY);
         std::swap(TmpPoint[2].textureX, TmpPoint[3].textureX);
      }
      rectangle.applyToCorners(TmpPoint[0], TmpPoint[1], TmpPoint[2], TmpPoint[3]);
      rectangle.setTexture(texture->parentTexture);
   }

   void AssignTextureToRectangle(DrawRectangle &rectangle, MainTexture *texture, bool flip){
      AssignTextureToRectangle(rectangle, &(texture->texture), flip);
   }

   void AssignTextureToRectangle(DrawRectangle &rectangle, GLuint *texture, bool flip){
      TexturePoint TmpPoint[4];
      TmpPoint[0].textureX = 0.0;   TmpPoint[0].textureY = 0.0;
      TmpPoint[1].textureX = 1.0;   TmpPoint[1].textureY = 0.0;
      TmpPoint[2].textureX = 1.0;   TmpPoint[2].textureY = 1.0;
      TmpPoint[3].textureX = 0.0;   TmpPoint[3].textureY = 1.0;

      if(flip){
         std::swap(TmpPoint[0].textureY, TmpPoint[1].textureY);
         std::swap(TmpPoint[0].textureX, TmpPoint[1].textureX);
         std::swap(TmpPoint[2].textureY, TmpPoint[3].textureY);
         std::swap(TmpPoint[2].textureX, TmpPoint[3].textureX);
      }
      rectangle.applyToCorners(TmpPoint[0], TmpPoint[1], TmpPoint[2], TmpPoint[3]);
      rectangle.setTexture(texture);
   }

   /*************************\
   | ---------Scene--------- |
   \*************************/

   Scene::Scene():drawSorted(true),isSorted(false){}

   Scene::Scene(Draw2D *context):DrawShape(context),drawSorted(true),isSorted(false){}

   Scene::~Scene(){
      clear();
   }

   void Scene::setRenderer( Draw2D* context ){
      renderer = context;
      std::map<std::string, DrawShape*>::iterator cell;
      for(cell = DrawList.begin();cell != DrawList.end();cell++){
         cell->second->setRenderer(context);
      }
   }

   void Scene::clear(){
      std::for_each(DrawList.begin(), DrawList.end(), [](std::pair<const std::string, DrawShape*> &cell){
         delete cell.second;
      });
      DrawList.clear();
   }

   DrawShape* Scene::add( DrawShape *childItem, const std::string &childId ){
      alertParent("NeedsSort");
      isSorted = 0;
      if(DrawList.find(childId) == DrawList.end()){
         childItem->setParent(this);
         if(renderer){
            childItem->setRenderer(renderer);
         }
         DrawList[childId] = childItem;
         return childItem;
      }
      return nullptr;
   }

   bool Scene::remove( DrawShape* childItem, bool deleteOnRemove ){
      std::map<std::string, DrawShape*>::iterator cell;
      for(cell = DrawList.begin();cell != DrawList.end() && cell->second != childItem;cell++){
         ;
      }
      if(cell->second == childItem){
         if(deleteOnRemove){
            delete(cell->second);
         }
         DrawList.erase(cell);
         return true;
      }
      return false;
   }

   bool Scene::remove(const std::string &childId, bool deleteOnRemove ){
      std::map<std::string, DrawShape*>::iterator cell = DrawList.find(childId);
      if(cell != DrawList.end()){
         if(deleteOnRemove){
            delete DrawList[childId];
         }
         DrawList.erase(cell);
         return true;
      }
      return false;
   }

   DrawShape* Scene::getChild(const std::string &childId){
      if(DrawList.find(childId) != DrawList.end()){
         return DrawList[childId];
      }
      return false;
   }

   void Scene::draw(){
      pushMatrix();
      if(drawSorted){
         sortedRender();
      }else{
         unsortedRender();
      }
      popMatrix();
   }

   void Scene::sortedRender(){
      if(!isSorted){
         DrawListVector.clear();
         std::for_each(DrawList.begin(), DrawList.end(), [&](std::pair<const std::string, DrawShape*> &cell){
            DrawListVector.push_back(cell.second);
         });
         std::sort(DrawListVector.begin(), DrawListVector.end(), [](DrawShape* one, DrawShape* two){
            return *one < *two;
         });
         isSorted = true;
      }
      std::for_each(DrawListVector.begin(), DrawListVector.end(), std::mem_fun(&DrawShape::draw));
   }

   void Scene::unsortedRender(){
      std::for_each(DrawList.begin(), DrawList.end(), [](std::pair<const std::string, DrawShape*> &cell){
         cell.second->draw();
      });
   }

   double Scene::getDepth(){
      int elements = 0;
      double total = 0;
      std::map<std::string, DrawShape*>::iterator cell;
      for(cell = DrawList.begin();cell != DrawList.end();cell++){
         elements++;
         total+=cell->second->getDepth();
      }
      total/=(double)elements;
      return total;
   }

   std::string Scene::getMessage(const std::string &message ){
      if(message == "NeedsSort"){
         alertParent(message);
         isSorted = 0;
      }else if(message == "pushMatrix"){
         alertParent(message);
         pushMatrix();
      }else if(message == "popMatrix"){
         popMatrix();
         alertParent(message);
      }
      return "";
   }

   void Scene::sortScene( bool depthMatters ){
      drawSorted = depthMatters;
   }

   void Scene::setColor( Color newColor ){
      std::map<std::string, DrawShape*>::iterator cell;
      for(cell = DrawList.begin();cell != DrawList.end();cell++){
         cell->second->setColor(newColor);
      }
   }

   void Scene::setColor( float R, float G, float B, float A ){
      Color tmpColor(R, G, B, A);
      std::map<std::string, DrawShape*>::iterator cell;
      for(cell = DrawList.begin();cell != DrawList.end();cell++){
         cell->second->setColor(R, G, B, A);
      }
   }

   BoxAABB Scene::getAABB(){
      BoxAABB tmpBox1, tmpBox2;
      bool first = true;
      std::map<std::string, DrawShape*>::iterator cell;
      for(cell = DrawList.begin();cell != DrawList.end();cell++){
         tmpBox1 = cell->second->getAABB();
         if(first){
            tmpBox2.initialize(tmpBox1);
            first = false;
         }else{
            tmpBox2.expandWith(tmpBox1);
         }
      }
      return tmpBox2;
   }

   /*************************\
   | ------PointVolume------ |
   \*************************/

   bool PointVolume::pointContained( double x, double y ){
      Point tmpPoint(x, y, 0.0);
      return pointContained(tmpPoint);
   }

   bool PointVolume::pointContained( const Point &comparePoint ){
      int i;
      double angle=0;
      Point p1,p2;
      int totalPoints = (int)points.size();
      for (i=0;i<totalPoints;i++) {
         p1.x = points[i].x - comparePoint.x;
         p1.y = points[i].y - comparePoint.y;
         p2.x = points[(i+1)%totalPoints].x - comparePoint.x;
         p2.y = points[(i+1)%totalPoints].y - comparePoint.y;
         angle += getAngle((float)p1.x,(float)p1.y,(float)p2.x,(float)p2.y);
      }
      if(angle < 0){angle*=-1;}
      if (angle < PIE){
         return false;
      }
      return true;
   }
   void PointVolume::addPoint( const Point &newPoint ){
      points.push_back(newPoint);
   }

   double PointVolume::getAngle( double x1, double y1, double x2, double y2 ){
      double dtheta,theta1,theta2;

      theta1 = atan2(y1,x1);
      theta2 = atan2(y2,x2);
      dtheta = theta2 - theta1;
      while (dtheta > PIE){
         dtheta -= PIE*2.0;
      }
      while (dtheta < -PIE){
         dtheta += PIE*2.0;
      }

      return(dtheta);
   }

   Point PointVolume::getCenter(){
      Point average;
      int totalPoints = (int)points.size();
      for (int i=0;i<totalPoints;i++) {
         average.x+=points[i].x;
         average.y+=points[i].y;
         average.z+=points[i].z;
      }
      average.x/=double(totalPoints);
      average.y/=double(totalPoints);
      average.z/=double(totalPoints);
      return average;
   }

   bool PointVolume::volumeCollision( PointVolume &compareVolume,  Draw2D* renderer){
      Point point1 = getCenter();
      Point point2 = compareVolume.getCenter();

      double angle = getAngle(point1.x, point1.y, point2.x, point2.y);
      angle = angle * (180 / PIE);
      angle+=90.0;

      PointVolume tmpVolume1, tmpVolume2;
      int totalPoints;
      glPushMatrix();
      glLoadIdentity();
      glRotated(angle, 0.0, 0.0, 1.0);

      totalPoints = (int)points.size();
      for(int i = 0;i < totalPoints;i++){
         tmpVolume1.addPoint(renderer->getObjectToWorldPoint(points[i]));
      }

      totalPoints = (int)compareVolume.points.size();
      for(int i = 0;i < totalPoints;i++){
         tmpVolume2.addPoint(renderer->getObjectToWorldPoint(compareVolume.points[i]));
      }
      glPopMatrix();

      BoxAABB box1 = tmpVolume1.getAABB();
      BoxAABB box2 = tmpVolume2.getAABB();

      if((box2.minPoint.x > box1.minPoint.x && box2.maxPoint.x > box1.maxPoint.x) || 
         (box2.minPoint.x < box1.minPoint.x && box2.maxPoint.x < box1.maxPoint.x)) {
         return false;
      }
      return true;
   }

   BoxAABB PointVolume::getAABB(){
      BoxAABB result;
      int totalPoints = (int)points.size();
      if(totalPoints > 0){
         result.initialize(points[0]);
         for(int i = 1;i < totalPoints;i++){
            result.expandWith(points[i]);
         }
      }
      return result;
   }

   /*************************\
   | -------WorldScene------ |
   \*************************/

   Scene* WorldScene::makeNewScene( std::string Name, bool NameIndicatesGroup ){
      Scene *newScene = new Scene(renderer);
      if(NameIndicatesGroup){
         if(groupIdList.find(Name) == groupIdList.end()){
            groupIdList[Name] = 0;
         }
         Name+=boost::lexical_cast<std::string>(groupIdList[Name]++);
      }
      recentName = Name;
      rootScene.add(newScene, Name);
      return newScene;
   }

   Scene* WorldScene::addScene( Scene* newScene, std::string Name, bool NameIndicatesGroup ){
      newScene->setRenderer(renderer);
      if(NameIndicatesGroup){
         if(groupIdList.find(Name) == groupIdList.end()){
            groupIdList[Name] = 0;
         }
         Name+=boost::lexical_cast<std::string>(groupIdList[Name]++);
      }
      recentName = Name;
      rootScene.add(newScene, Name);
      return newScene;
   }

   Scene* WorldScene::getScene( const std::string &Name ){
      DrawShape* item = rootScene.getChild(Name);
      if(item){
         return item->getThis<Scene>();
      }
      return 0;
   }

   void WorldScene::draw(){
      glPushMatrix();
      glTranslated(-viewLocation.x, -viewLocation.y, -viewLocation.z);
      rootScene.draw();
      glPopMatrix();
   }

   void WorldScene::clear(){
      groupIdList.clear();
      rootScene.clear();
   }

   bool WorldScene::removeScene( const std::string &Name ){
      return rootScene.remove(Name);
   }

   bool WorldScene::removeScene( Scene* delScene ){
      return rootScene.remove(delScene);
   }

   std::string WorldScene::getNewSceneName(){
      return recentName;
   }
}
