#include "points.h"
namespace M2Rend {
   /*************************\
   | ------TexturePoint----- |
   \*************************/

   TexturePoint& TexturePoint::operator=( const TexturePoint& other ){
      textureX = other.textureX; textureY = other.textureY;
      return *this;
   }

   TexturePoint& TexturePoint::operator=( const DrawPoint& other ){
      textureX = other.textureX; textureY = other.textureY;
      return *this;
   }

   /*************************\
   | ---------Color--------- |
   \*************************/

   Color& Color::operator=( const Color& other ){
      R = other.R; G = other.G; B = other.B; A = other.A;
      return *this;
   }

   Color& Color::operator=( const DrawPoint& other ){
      R = other.R; G = other.G; B = other.B; A = other.A;
      return *this;
   }

   /*************************\
   | ---------Point--------- |
   \*************************/

   void Point::clear(){
      x = 0; y = 0; z = 0;
   }

   bool Point::operator+=(const Point& other){
      x+=other.x; y+=other.y; z+=other.z;
      return true;
   }

   bool Point::operator-=(const Point& other){
      x-=other.x; y-=other.y; z-=other.z;
      return true;
   }

   bool Point::operator*=(const Point& other){
      x*=other.x; y*=other.y; z*=other.z;
      return true;
   }

   bool Point::operator*=(const double& other){
      x*=other; y*=other; z*=other;
      return true;
   }

   bool Point::operator/=(const Point& other){
      x/=other.x; y/=other.y; z/=other.z;
      return true;
   }

   bool Point::operator/=(const double& other){
      x/=other; y/=other; z/=other;
      return true;
   }

   Point Point::translate( double xAmount, double yAmount, double zAmount ){
      x+=xAmount; y+=yAmount; z+=zAmount;
      return *this;
   }

   Point Point::translate( double xAmount, double yAmount ){
      x+=xAmount; y+=yAmount;
      return *this;
   }

   Point Point::placeAt( double xPos, double yPos, double zPos ){
      x = xPos; y = yPos; z = zPos;
      return *this;
   }

   Point Point::placeAt( double xPos, double yPos ){
      x = xPos; y = yPos;
      return *this;
   }

   Point Point::scale( double amount ){
      x*=amount; y*=amount; z*=amount;
      return *this;
   }

   Point operator+(const Point& left, const Point& right){
      Point tmpPoint = left;
      tmpPoint+=right;
      return tmpPoint;
   }

   Point operator-(const Point& left, const Point& right){
      Point tmpPoint = left;
      tmpPoint-=right;
      return tmpPoint;
   }

   Point operator*(const Point& left, const Point& right){
      Point tmpPoint = left;
      tmpPoint*=right;
      return tmpPoint;
   }

   Point operator/(const Point& left, const Point& right){
      Point tmpPoint = left;
      tmpPoint/=right;
      return tmpPoint;
   }

   Point operator*(const Point& left, const double& right){
      Point tmpPoint = left;
      tmpPoint*=right;
      return tmpPoint;
   }

   Point operator/(const Point& left, const double& right){
      Point tmpPoint = left;
      tmpPoint/=right;
      return tmpPoint;
   }

   /*************************\
   | -------DrawPoint------- |
   \*************************/

   DrawPoint& DrawPoint::operator=( const DrawPoint& other ){
      x = other.x; y = other.y; z = other.z;
      textureX = other.textureX; textureY = other.textureY;
      R = other.R; G = other.G; B = other.B; A = other.A;
      return *this;
   }

   DrawPoint& DrawPoint::operator=( const Point& other ){
      x = other.x; y = other.y; z = other.z;
      return *this;
   }

   DrawPoint& DrawPoint::operator=( const Color& other ){
      R = other.R; G = other.G; B = other.B; A = other.A;
      return *this;
   }

   DrawPoint& DrawPoint::operator=( const TexturePoint& other ){
      textureX = other.textureX; textureY = other.textureY;
      return *this;
   }

   DrawPoint DrawPoint::operator+(DrawPoint& other){
      DrawPoint returnPoint;
      returnPoint.x = x+other.x;   returnPoint.y = x+other.y;   returnPoint.z = x+other.z;
      returnPoint.textureX = textureX+other.textureX;   returnPoint.textureY = textureY+other.textureY;
      returnPoint.R = R+other.R;   returnPoint.G = G+other.G; returnPoint.B = B+other.B;
      returnPoint.A = A+other.A;
      return returnPoint;
   }
   DrawPoint DrawPoint::operator-(DrawPoint& other){
      DrawPoint returnPoint;
      returnPoint.x = x-other.x;   returnPoint.y = x-other.y;   returnPoint.z = x-other.z;
      returnPoint.textureX = textureX-other.textureX;   returnPoint.textureY = textureY-other.textureY;
      returnPoint.R = R-other.R;   returnPoint.G = G-other.G; returnPoint.B = B-other.B;
      returnPoint.A = A-other.A;
      return returnPoint;
   }

   bool DrawPoint::operator+=(DrawPoint& other){
      x+=other.x; y+=other.y; z+=other.z;
      textureX+=other.textureX; textureY+=other.textureY;
      R+=other.R; G+=other.G; B+=other.B; A+=other.A;
      return true;
   }

   bool DrawPoint::operator-=(DrawPoint& other){
      x-=other.x; y-=other.y; z-=other.z;
      textureX-=other.textureX; textureY-=other.textureY;
      R-=other.R; G-=other.G; B-=other.B; A-=other.A;
      return true;
   }

   void DrawPoint::clear(){
      x = 0; y = 0; z = 0;
      textureX = 0; textureY = 0;
      R = 1.0; G = 1.0; B = 1.0; A = 1.0;
   }

   void DrawPoint::copyPosition( DrawPoint& other ){
      x = other.x; y = other.y; z = other.z;
   }

   void DrawPoint::copyTexture( DrawPoint& other ){
      textureX = other.textureX; textureY = other.textureY;
   }

   void DrawPoint::copyColor( DrawPoint& other ){
      R = other.R; G = other.G; B = other.B;
   }

}
