#ifndef __POINTS_H__
#define __POINTS_H__

namespace M2Rend {

   class DrawPoint;

   class TexturePoint{
   public:
      TexturePoint(){textureX = 0.0; textureY = 0.0;}
      ~TexturePoint(){}

      TexturePoint& operator=(const TexturePoint& other);
      TexturePoint& operator=(const DrawPoint& other);

      double textureX, textureY;
   };

   class Color{
   public:
      Color(){R = 1.0; G = 1.0; B = 1.0; A = 1.0;}
      Color(float Red, float Green, float Blue, float Alpha = 1.0){R = validRange(Red); G = validRange(Green); B = validRange(Blue); A = validRange(Alpha);}
      ~Color(){}
      Color& operator=(const Color& other);
      Color& operator=(const DrawPoint& other);

      float R, G, B, A;
   private:
      float validRange(float color){
         if(color > 1.0){color = 1.0;}
         if(color < 0.0){color = 0.0;}
         return color;
      }
   };

   class Point{
   public:
      Point(){clear();}
      Point(double xPos, double yPos, double zPos = 0.0){placeAt(xPos, yPos, zPos);}
      
      virtual void clear();
      
      Point scale(double amount);
      Point placeAt(double xPos, double yPos, double zPos);
      Point placeAt(double xPos, double yPos);
      Point translate(double xAmount, double yAmount, double zAmount);
      Point translate(double xAmount, double yAmount);
      
      bool atOrigin(){return x == 0 && y == 0 && z == 0;}
      
      bool operator+=(const Point& other);
      bool operator-=(const Point& other);
      bool operator*=(const Point& other);
      bool operator/=(const Point& other);
      bool operator*=(const double& other);
      bool operator/=(const double& other);

      double x, y, z;
   };

   Point operator+(const Point& left, const Point& right);
   Point operator-(const Point& left, const Point& right);
   Point operator*(const Point& left, const Point& right);
   Point operator/(const Point& left, const Point& right);
   Point operator*(const Point& left, const double& right);
   Point operator/(const Point& left, const double& right);

   class DrawPoint : public Point, public Color, public TexturePoint{
   public:
      DrawPoint(){clear();}
      DrawPoint(double xPos, double yPos, double zPos){placeAt(xPos, yPos, zPos);}
      ~DrawPoint(){}

      virtual void clear();

      DrawPoint& operator=(const DrawPoint& other);
      DrawPoint& operator=(const Point& other);
      DrawPoint& operator=(const Color& other);
      DrawPoint& operator=(const TexturePoint& other);

      DrawPoint operator+(DrawPoint& other);
      DrawPoint operator-(DrawPoint& other);
      bool operator+=(DrawPoint& other);
      bool operator-=(DrawPoint& other);

      void copyPosition(DrawPoint& other);
      void copyTexture(DrawPoint& other);
      void copyColor(DrawPoint& other);
   };

}
#endif
