#include "render.h"

namespace M2Rend {
   /*************************\
   | ------Extensions------- |
   \*************************/

   bool glExtensions::loadExtensions(){
      char* extensionsList = (char*) glGetString(GL_EXTENSIONS);
      bool returnVal = 1;

      returnVal&=loadBlendFuncSeparate(extensionsList);

      return returnVal;
   }

   void glExtensions::setBlendMode( GLenum sfactorRGB, GLenum dfactorRGB, GLenum sfactorAlpha, GLenum dfactorAlpha ){
      if(pglBlendFuncSeparateEXT!=NULL){
         pglBlendFuncSeparateEXT(sfactorRGB, dfactorRGB, sfactorAlpha, dfactorAlpha);
      }else{
         glBlendFunc(sfactorRGB, dfactorRGB);
      }
   }

   bool glExtensions::loadBlendFuncSeparate(char* extensionsList){
      if(pglBlendFuncSeparateEXT==NULL){
         if(strstr(extensionsList, "GL_EXT_blend_func_separate ")==NULL){
            std::cerr << "\nError: The OpenGL extension GL_EXT_blend_func_separate IS NOT SUPPORTED ON THIS SYSTEM\n"  << std::endl;
            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
            return false;
         }else{  
            pglBlendFuncSeparateEXT = ( void (APIENTRY*) (GLenum, GLenum, GLenum, GLenum)) SDL_GL_GetProcAddress("glBlendFuncSeparateEXT");
            pglBlendFuncSeparateEXT(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA, GL_ONE_MINUS_DST_ALPHA, GL_ONE);
         }
      }
      return true;
   }

   /*************************\
   | --------Draw2D--------- |
   \*************************/

   Draw2D::Draw2D() : backgroundColor(0.0, 0.0, 0.0, 0.0), windowTitle("M2tM Application"), initialized(0), windowIsFullScreen(0) {
      SDLflags=SDL_OPENGL|SDL_ANYFORMAT|SDL_SRCALPHA;
   }

   void Draw2D::allowWindowResize( bool allowResize ){
      if(allowResize){
         SDLflags = SDLflags | SDL_RESIZABLE;
      }else{
         SDLflags = SDLflags & ~ SDL_RESIZABLE;
      }
   }

   void Draw2D::useFullScreen( bool isFullScreen ){
      if(isFullScreen){
         SDLflags = SDLflags | SDL_FULLSCREEN;
      }else{
         SDLflags = SDLflags & ~ SDL_FULLSCREEN;
      }
   }

   bool Draw2D::initialize( int windowsWidth, int windowsHeight, double drawingWidth, double drawingHeight){
      windowWidth = windowsWidth; windowHeight = windowsHeight;
      if(drawingWidth == -1 || drawingHeight == -1){
         worldWidth = windowWidth; worldHeight = windowHeight;
      }else{
         worldWidth = drawingWidth; worldHeight = drawingHeight;
      }
      if(setupSDL()){
         setupOpengl();
         if(loadExtensions()){
            return 1;
         }
      }
      return 0;
   }

   bool Draw2D::setupSDL( ){
      if( SDL_Init(SDL_INIT_VIDEO) < 0 ){
         // Failed, exit
         std::cerr << "Video initialization failed: " << SDL_GetError() << std::endl;
         return(0);
      }
      windowIsFullScreen = 0;
      if(SDLflags&SDL_FULLSCREEN){
         windowIsFullScreen = 1;
         SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
      }
      SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 8 );
      SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 8 );
      SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 8 );
      SDL_GL_SetAttribute( SDL_GL_ALPHA_SIZE, 8 );

      SDL_GL_SetAttribute( SDL_GL_ACCUM_RED_SIZE, 16 );
      SDL_GL_SetAttribute( SDL_GL_ACCUM_GREEN_SIZE, 16 );
      SDL_GL_SetAttribute( SDL_GL_ACCUM_BLUE_SIZE, 16 );
      SDL_GL_SetAttribute( SDL_GL_ACCUM_ALPHA_SIZE, 16 );

      SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
      SDL_GL_SetAttribute( SDL_GL_SWAP_CONTROL, 0 ); //disable or enable vsync
      SDL_WM_SetCaption(windowTitle.c_str(), windowTitle.c_str());
      if( SDL_SetVideoMode( windowWidth, windowHeight, 32, SDLflags) == NULL ){
         std::cerr << "Video mode set failed: " << SDL_GetError() << std::endl;
         atexit(SDL_Quit); // Quit SDL at exit.
         return(0);
      }
      initialized = 1;
      return 1;
   }

   void Draw2D::setupOpengl( ){
      glShadeModel( GL_SMOOTH );

      glMatrixMode(GL_PROJECTION);
      glLoadIdentity();
      glViewport( 0, 0, windowWidth, windowHeight );
      glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

      glOrtho(0, worldWidth, worldHeight, 0, -128.0f, 128.0f);

      glMatrixMode(GL_MODELVIEW);
      glLoadIdentity();

      glClearColor(backgroundColor.R,backgroundColor.G,backgroundColor.B,0.0f);
      glClearDepth(15.0f);

      glEnable (GL_BLEND);
      glDisable (GL_ALPHA_TEST);

      glDisable(GL_DEPTH_TEST);
      glShadeModel(GL_SMOOTH);
      glDisable(GL_CULL_FACE);
      glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
      glHint(GL_SMOOTH, GL_NICEST);
      glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
      glDepthMask(0);

      glEnable(GL_TEXTURE_2D);
      glDisable(GL_LIGHTING);
      glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   }

   bool Draw2D::resizeWindow( int x, int y ){
      if(x > 0 && y > 0){
         windowWidth = x; windowHeight = y;
      }
      int fullScreenResult = fullScreenTransition();
      if(fullScreenResult==-1){
         if(SDL_SetVideoMode(x, y, 32, SDLflags)==NULL){
            return 0;
         }else{
            setupOpengl();
            return 1;
         }
      }
      return fullScreenResult != 0;
   }

   void Draw2D::resizeWorldSpace( double worldsWidth, double worldsHeight ){
      worldWidth = worldsWidth; worldHeight = worldsHeight;
      setupOpengl();
   }

   int Draw2D::fullScreenTransition(){
      if(!windowIsFullScreen && (SDLflags & SDL_FULLSCREEN)){
         SDL_QuitSubSystem(SDL_INIT_VIDEO);
         if(setupSDL()){
            setupOpengl();
            return 1;
         }
         return 0;
      }else if(windowIsFullScreen && !(SDLflags & SDL_FULLSCREEN)){
         SDL_QuitSubSystem(SDL_INIT_VIDEO);
         if(setupSDL()){
            setupOpengl();
            return 1;
         }
         return 0;
      }
      return -1;
   }

   bool Draw2D::isWindowFullScreen(){
      return windowIsFullScreen;
   }

   void Draw2D::setBlendMode( GLenum sfactorRGB, GLenum dfactorRGB ){
      glBlendFunc(sfactorRGB, dfactorRGB);
   }

   void Draw2D::clearScreen(){
      glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   }

   void Draw2D::updateScreen(){
      SDL_GL_SwapBuffers( );
   }

   void Draw2D::setWindowTitle( std::string title ){
      windowTitle = title;
      if(initialized){
         SDL_WM_SetCaption(windowTitle.c_str(), windowTitle.c_str());
      }
   }

   Point Draw2D::getObjectToWorldPoint( double worldX, double worldY ){
      GLdouble model_view[16], projection[16];
      GLint viewport[4];
      Point tmpPoint;
      glGetDoublev(GL_MODELVIEW_MATRIX, model_view);
      glGetDoublev(GL_PROJECTION_MATRIX, projection);
      glGetIntegerv(GL_VIEWPORT, viewport);
      gluProject(worldX, worldY, 0.0, model_view, projection, viewport, &tmpPoint.x, &tmpPoint.y, &tmpPoint.z);
      //Re-orient for our actual window position based on 0, 0 as top left instead of bottom left.
      tmpPoint.y-=windowHeight;
      tmpPoint.y*=-1.0;
      //Finally, retrieve the actual location of the object in the world by taking into account the window dimensions
      //in relation to the screen dimensions.
      double scaleDifferenceY = worldHeight/(double)windowHeight, scaleDifferenceX = worldWidth/(double)windowWidth;

      tmpPoint.x*=scaleDifferenceX;
      tmpPoint.y*=scaleDifferenceY;

      return tmpPoint;
   }

   Point Draw2D::getObjectToWorldPoint( Point worldPoint ){
      return getObjectToWorldPoint(worldPoint.x, worldPoint.y);
   }

   Point Draw2D::getWindowToWorldPoint( double windowX, double windowY ){
      GLdouble model_view[16], projection[16];
      GLint viewport[4];
      Point tmpPoint;
      glGetDoublev(GL_MODELVIEW_MATRIX, model_view);
      glGetDoublev(GL_PROJECTION_MATRIX, projection);
      glGetIntegerv(GL_VIEWPORT, viewport);
      gluProject(windowX, windowY, 0.0, model_view, projection, viewport, &tmpPoint.x, &tmpPoint.y, &tmpPoint.z);
      //Re-orient for our actual window position based on 0, 0 as top left instead of bottom left.
      tmpPoint.y-=windowHeight;
      tmpPoint.y*=-1.0;
      //Finally, retrieve the actual location of the object in the world by taking into account the window dimensions
      //in relation to the screen dimensions.
      double scaleDifferenceY = worldHeight/(double)windowHeight, scaleDifferenceX = worldWidth/(double)windowWidth;

      tmpPoint.x*=scaleDifferenceX;
      tmpPoint.y*=scaleDifferenceY;

      return tmpPoint;
   }

   Point Draw2D::getWindowToWorldPoint( Point windowPoint ){
      return getWindowToWorldPoint(windowPoint.x, windowPoint.y);
   }

   Point Draw2D::getWorldSize(){
      Point tmpPoint;
      tmpPoint.x = worldWidth;
      tmpPoint.y = worldHeight;
      return tmpPoint;
   }

   Point Draw2D::getWindowSize(){
      Point tmpPoint;
      tmpPoint.x = windowWidth;
      tmpPoint.y = windowHeight;
      return tmpPoint;
   }

   void Draw2D::setBackgroundColor( Color newColor ){
      backgroundColor = newColor;
      glClearColor(backgroundColor.R,backgroundColor.G,backgroundColor.B,0.0f);
   }
}
