/**********************************************************\
| Michael Hamilton (maxmike@gmail.com) www.mutedvision.net |
|----------------------------------------------------------|
\**********************************************************/

#ifndef _RENDER_H_
#define _RENDER_H_

#include <algorithm>
#include <functional>
#include "Render/points.h"
#include "SDL/SDL.h"
#include "SDL/SDL_opengl.h"
#include <string>
#include <iostream>

namespace M2Rend {

   #if SDL_BYTEORDER == SDL_BIG_ENDIAN
     const Uint32 SDL_RMASK = 0xff000000;
     const Uint32 SDL_GMASK = 0x00ff0000;
     const Uint32 SDL_BMASK = 0x0000ff00;
     const Uint32 SDL_AMASK = 0x000000ff;
   #else
     const Uint32 SDL_RMASK = 0x000000ff;
     const Uint32 SDL_GMASK = 0x0000ff00;
     const Uint32 SDL_BMASK = 0x00ff0000;
     const Uint32 SDL_AMASK = 0xff000000;
   #endif

   #ifndef GLNULL
      #define GLNULL 0
   #endif
   #if defined(WIN32) && !defined(GL_BGR)
      #define GL_BGR GL_BGR_EXT
   #endif

   class glExtensions {
   public:
      glExtensions(){
         pglBlendFuncSeparateEXT = NULL;
      }
      ~glExtensions(){}

      void setBlendMode(GLenum sfactorRGB, GLenum dfactorRGB, GLenum sfactorAlpha, GLenum dfactorAlpha);

   protected:
      bool loadExtensions();

      //Extensions
      void (APIENTRY * pglBlendFuncSeparateEXT) (GLenum, GLenum, GLenum, GLenum);
   private:
      bool loadBlendFuncSeparate(char* extensionsList);
   };

   class Draw2D : public glExtensions {
   public:
      Draw2D();
      ~Draw2D(){}

      void setWindowTitle(std::string title);

      //Warning: Any time the window is resized the OpenGL context is trashed and all textures must be re-loaded.
      void allowWindowResize(bool allowResize);
      void useFullScreen(bool isFullScreen);
      bool initialize(int windowsWidth, int windowsHeight, double worldsWidth = -1, double worldsHeight = -1);

      void setBackgroundColor(Color newColor);
      void setBlendMode(GLenum sfactorRGB, GLenum dfactorRGB);
      bool resizeWindow(int x = -1, int y = -1);
      void resizeWorldSpace(double drawingWidth, double drawingHeight);
      void clearScreen();
      void updateScreen();

      Point getWindowSize();
      Point getWorldSize();

      //Given a point x, y return the position in world space where that will be rendered
      //within the current projection/modelview.  Gets the final world point after transformations
      //from an object's local points.
      Point getObjectToWorldPoint(double objectX, double objectY);
      Point getObjectToWorldPoint(Point worldPoint);

      Point getWindowToWorldPoint(double windowX, double windowY);
      Point getWindowToWorldPoint(Point windowPoint);

      bool isWindowFullScreen();
   private:
      int fullScreenTransition();
      bool setupSDL();
      void setupOpengl();

      Color backgroundColor;
      std::string windowTitle;
      int SDLflags;
      int windowWidth, windowHeight;
      double worldWidth, worldHeight;
      bool windowIsFullScreen, initialized;
   };
}
#endif
