#ifndef __TEXT_H__
#define __TEXT_H__

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "Utility/package.h"
#include "SDL/SDL.h"
#include "SDL/SDL_ttf.h"
#include "SDL/SDL_opengl.h"
#include "drawShapes.h"

namespace M2Rend {
   class TextCharacter {
   public:
      TextCharacter():set(false),textureId(0),character(' '){}
      ~TextCharacter(){}

      void assignSDLSurface(SDL_Surface *newSurface, char glyphChar);

      bool isSet();

      char getChar();
      GLuint* getTextureId();
      Point getCharacterSize();
      Point getTextureSize();
   private:
      void assignSurfaceToOpenGL(SDL_Surface *newSurface, int widthPowerOfTwo, int heightPowerOfTwo);
      void generateTexture(SDL_Surface *surface);

      bool set;
      char character;
      Point size, textureSize;
      GLuint textureId;
   };


   class TextLibrary{
   public:
      TextLibrary(M2Rend::Draw2D *rendering);
      ~TextLibrary();

      bool loadFont(std::string identifier, int pointSize, std::string fontFileLocation);

      M2Rend::Scene* composeScene(std::string identifier, std::string text);

      //Call this when the OpenGL context is destroyed to re-load the textures.
      //Needs work, not presently functioning.
      void reloadTextures();
   private:
      std::vector<TextCharacter>* initGlyphs(const std::string &identifier, const std::string &text);
      std::vector<TextCharacter>* createGlyphContainer(const std::string &identifier);
      void loadIndividualGlyphs(const std::string &identifier, const std::string &text, std::vector<TextCharacter> &characterList);

      std::map<std::string, TTF_Font*> loadedFonts;
      std::map<std::string, std::vector<TextCharacter>> cachedGlyphs;
      SDL_Color white;
      M2Rend::Draw2D *render;
   };
}

#endif
