#include "textures.h"
#include <iostream>

namespace M2Rend {
   TextureManager::TextureManager(){
   }

   TextureManager::~TextureManager(){
   }

   GLuint TextureManager::createEmptyTexture(const std::string &name, unsigned int width, unsigned int height){
      if (width < 1 || height < 1) {
         return GLNULL;
      } else if ( !((width & (width-1))==0) || !((height & (height-1))==0)) {
         return GLNULL;
      }
      GLuint txtnumber;						// Texture ID
      unsigned int* data;						// Stored Data
      unsigned int imageSize = (width * height)* 4 * sizeof(unsigned int);
      // Create Storage Space For Texture Data (128x128x4)
      data = (unsigned int*)new GLuint[(imageSize)];
      memset(data, 0,(imageSize));
      glGenTextures(1, &txtnumber);					// Create 1 Texture
      glBindTexture(GL_TEXTURE_2D, txtnumber);			// Bind The Texture
      // Build Texture Using Information In data
      glTexImage2D(GL_TEXTURE_2D, 0, 4, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, data);
      glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
      glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);

      delete [] data;							// Release data
      MainTexture tmp;
      tmp.doNotSave = 1;
      tmp.file = "none";
      tmp.height = height;
      tmp.width = width;
      tmp.name = name;
      tmp.texture = txtnumber;
      mainTextures[name] = tmp;
      return txtnumber;						// Return The Texture ID
   }


   bool TextureManager::deleteSubTexture(const std::string &mainName, const std::string &subName){
      MainTexture *tmpMain = getMainTexture(mainName);
      if(tmpMain && (tmpMain->subTextures.find(subName) != tmpMain->subTextures.end())){
         tmpMain->subTextures.erase(subName);
         return 1;
      }
      return 0;
   }
   bool TextureManager::deleteSubTexture(int mainNum, const std::string &subName){
      MainTexture *tmpMain = getMainTexture(mainNum);
      if(tmpMain && (tmpMain->subTextures.find(subName) != tmpMain->subTextures.end())){
         tmpMain->subTextures.erase(subName);
         return 1;
      }
      return 0;
   }
   bool TextureManager::deleteSubTexture(const std::string &mainName, int subNum){
      SubTexture *tmpSub = getSubTexture(mainName, subNum);
      if(tmpSub){
         MainTexture *tmpMain = getMainTexture(mainName);
         tmpMain->subTextures.erase(tmpSub->name);
         return 1;
      }
      return 0;
   }
   bool TextureManager::deleteSubTexture(int mainNum, int subNum){
      MainTexture *tmpMain = getMainTexture(mainNum);
      SubTexture *tmpSub = getSubTexture(mainNum, subNum);
      if(tmpMain->subTextures.find(tmpSub->name) != tmpMain->subTextures.end()){
         tmpMain->subTextures.erase(tmpSub->name);
         return 1;
      }
      return 0;
   }

   bool TextureManager::loadTexture(const std::string &mainName, const std::string &file, bool repeat){
      MainTexture txtToAdd;
      txtToAdd.clamped = repeat;
      txtToAdd.name = mainName;
      txtToAdd.file = file;
      txtToAdd.doNotSave = 0;
      //Create the texture
      bool success = loadTextureInternal(file, txtToAdd.texture, txtToAdd.width, txtToAdd.height, repeat);

      if(success){
         mainTextures[mainName] = txtToAdd;
         return 1;
      }
      return 0;
   }

   bool TextureManager::reloadTexture(MainTexture &ourTexture){
      //Create the texture
      bool success = loadTextureInternal(ourTexture.file, ourTexture.texture, ourTexture.width, ourTexture.height, ourTexture.clamped);

      if(success){
         return 1;
      }
      return 0;
   }

   Uint32 TextureManager::getpixel(SDL_Surface *surface, int x, int y)
   {
      int bpp = surface->format->BytesPerPixel;
      /* Here p is the address to the pixel we want to retrieve */
      Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

      switch(bpp) {
       case 1:
          return *p;

       case 2:
          return *(Uint16 *)p;

       case 3:
          if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
             return p[0] << 16 | p[1] << 8 | p[2];
          else
             return p[0] | p[1] << 8 | p[2] << 16;

       case 4:
          return *(Uint32 *)p;

       default:
          return 0;       /* shouldn't happen, but avoids warnings */
      }
   }

   //Load an opengl texture
   bool TextureManager::loadTextureInternal(const std::string &file, GLuint &imageLoaded, int &w, int &h, bool clampIt) {
      bool alpha = 1;
      GLenum texture_format;
      bool repeat = clampIt;
      bool mipmaps = 1;
      int nOfColors;
      // Load the 'file' to SDL_Surface
      SDL_Surface *img = NULL;
      img = IMG_Load(file.c_str());
      if(img == NULL){
         return 0;
      }

      // Build the texture from the surface
      w = img->w;
      h = img->h; 

      //Check that the image's width is valid and then check that the image's width is a power of 2
      if (w < 1) {
         return 0;
      } else if ( !((w & (w-1))==0) ) {
         return 0;
      } 
      if (h < 1) {
         return 0;
      } else if ( !((h & (h-1))==0) ) {
         return 0;
      } 

      nOfColors = img->format->BytesPerPixel;
      if (nOfColors == 4)     // contains an alpha channel
      {
         if (img->format->Rmask == 0x000000ff)
            texture_format = GL_RGBA;
         else
            texture_format = GL_BGRA;
      } else if (nOfColors == 3)     // no alpha channel
      {
         if (img->format->Rmask == 0x000000ff)
            texture_format = GL_RGB;
         else
            texture_format = GL_BGR;
      } else {
         if ( img ) { 
            SDL_FreeSurface( img );
         }
         return 0;
      }

      int type = (alpha) ? GL_RGBA : GL_RGB;
      glGenTextures(1, &imageLoaded);		// Generate texture ID
      glBindTexture(GL_TEXTURE_2D, imageLoaded);

      glTexImage2D(GL_TEXTURE_2D, 0, nOfColors, img->w, img->h, 0, texture_format, GL_UNSIGNED_BYTE, img->pixels);

      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, (repeat) ? GL_REPEAT : GL_CLAMP_TO_EDGE);
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, (repeat) ? GL_REPEAT : GL_CLAMP_TO_EDGE);

      if(mipmaps){
         gluBuild2DMipmaps(GL_TEXTURE_2D, nOfColors, img->w, img->h, texture_format, GL_UNSIGNED_BYTE, img->pixels);
      }

      //Clean up and return the texture ID
      if ( img ) { 
         SDL_FreeSurface( img );
      }
      return 1;
   }

   MainTexture *TextureManager::getMainTexture(int mainNum){
      std::map<std::string, MainTexture>::iterator node = mainTextures.begin();
      for(;mainNum > 0 && node != mainTextures.end();mainNum--){
         node++;
      }
      if(node!=mainTextures.end()){
         return &(node->second);
      }
      return NULL;
   }

   MainTexture *TextureManager::getMainTexture(const std::string &mainName){
      if((mainTextures.find(mainName) != mainTextures.end())){
         return &mainTextures[mainName];
      }else{
         return NULL;
      }
   }

   GLuint *TextureManager::getMainTextureGLuint(const std::string &mainName){
      MainTexture *tmp = getMainTexture(mainName);
      if(tmp){
         return &tmp->texture;
      }else{
         return NULL;
      }
   }

   bool TextureManager::addSubTexture(const std::string &mainName, SubTexture &subTextureToAdd){
      MainTexture *tmp = getMainTexture(mainName);
      if(!tmp){
         return 0;
      }
      subTextureToAdd.percentWidth = float(float(subTextureToAdd.width)/float(tmp->width));
      subTextureToAdd.percentHeight = float(float(subTextureToAdd.height)/float(tmp->height));
      subTextureToAdd.percentLeftX = float(float(subTextureToAdd.leftX)/float(tmp->width));
      subTextureToAdd.percentTopY = float(float(subTextureToAdd.topY)/float(tmp->height));
      subTextureToAdd.mainHeight = tmp->height;
      subTextureToAdd.mainWidth = tmp->width;
      subTextureToAdd.parentTexture = &tmp->texture;
      tmp->subTextures[subTextureToAdd.name]=subTextureToAdd;
      return 1;
   }

   bool TextureManager::addSubTexture(const std::string &mainName, const std::string &subName, int leftX, int topY, int width, int height){
      SubTexture subTextureToAdd;
      subTextureToAdd.name = subName;
      subTextureToAdd.leftX = leftX;
      subTextureToAdd.topY = topY;
      subTextureToAdd.width = width;
      subTextureToAdd.height = height;
      return addSubTexture(mainName, subTextureToAdd);
   }

   SubTexture *TextureManager::getSubTexture(const std::string &mainName, const std::string &subName){
      MainTexture *tmp = getMainTexture(mainName);
      if(tmp && (tmp->subTextures.find(subName) != tmp->subTextures.end())){
         return &(tmp->subTextures[subName]);
      }
      return NULL;
   }

   GLuint *TextureManager::getSubTextureGLuint(const std::string &mainName, const std::string &subName){
      SubTexture *tmp = getSubTexture(mainName, subName);
      if(tmp){
         return tmp->parentTexture;
      }else{
         return NULL;
      }
   }

   //save the current state of the texture manager to a file
   bool TextureManager::save(const std::string &file){
      std::ofstream saveFile(file.c_str());
      saveFile << "http://www.mutedvision.net M2tM Texture Manager File" << std::endl;
      unsigned int i, j;
      if(mainTextures.size() == 0){
         return 0;
      }
      MainTexture *current;
      SubTexture *currentSub;
      for(i = 0;i < mainTextures.size();i++){
         current = getMainTexture(i);
         if(!current->doNotSave){
            saveFile << "Name: " << current->name << std::endl;
            saveFile << "File: " << current->file << std::endl;
            saveFile << "_" << std::endl;
            for(j = 0;j < current->subTextures.size();j++){
               currentSub = getSubTexture(*current, j);
               saveFile << "SubName: " << currentSub->name << std::endl;
               saveFile << "SubLeftX: " << currentSub->leftX << std::endl;
               saveFile << "SubTopY: " << currentSub->topY << std::endl;
               saveFile << "SubWidth: " << currentSub->width << std::endl;
               saveFile << "SubHeight: " << currentSub->height << std::endl;
               saveFile << "__" << std::endl;
            }
            saveFile << "___" << std::endl;
         }
      }
      saveFile << "____" << std::endl;
      return 1;
   }
   //load a file holding the state of a texture manager.  Loads all images and such
   //into opengl as well.
   bool TextureManager::load(const std::string &file){
      MainTexture mainText;
      SubTexture subText;
      mainText.name = "";
      bool valid = 0;
      subText.name = "";
      std::string stTmp;
      std::ifstream loadFile(file.c_str());
      if(loadFile.is_open()){
         loadFile.ignore(1024, '\n');
         while(stTmp!="____" && !loadFile.fail()){
            loadFile >> stTmp;
            if(stTmp == "Name:"){
               loadFile >> mainText.name;
            }else if(stTmp == "File:"){
               loadFile >> mainText.file;
            }else if(stTmp == "_"){
               valid = loadTexture(mainText.name, mainText.file);
            }else if(stTmp == "SubName:"){
               loadFile >> subText.name;
            }else if(stTmp == "SubLeftX:"){
               loadFile >> subText.leftX;
            }else if(stTmp == "SubTopY:"){
               loadFile >> subText.topY;
            }else if(stTmp == "SubWidth:"){
               loadFile >> subText.width;
            }else if(stTmp == "SubHeight:"){
               loadFile >> subText.height;
            }else if(stTmp == "__" && mainText.name != "" && subText.name != ""){
               valid = addSubTexture(mainText.name, subText.name, subText.leftX, subText.topY, subText.width, subText.height); 
               subText.name = "";
            }
         }
         return 1;
      }
      return 0;
   }

   int TextureManager::getNumSubTextures(const std::string &mainName){
      MainTexture *tmpTxt = getMainTexture(mainName);
      if(tmpTxt){
         return (int)(tmpTxt->subTextures.size());
      }
      return 0;
   }
   int TextureManager::getNumSubTextures(int mainNum){
      MainTexture *tmpTxt = getMainTexture(mainNum);
      if(tmpTxt){
         return (int)(tmpTxt->subTextures.size());
      }
      return 0;
   }
   SubTexture *TextureManager::getSubTexture(int mainNum, int subNum){
      MainTexture *tmpTxt = getMainTexture(mainNum);
      if(tmpTxt){
         return getSubTexture(tmpTxt->name, subNum);
      }
      return NULL;
   }
   SubTexture *TextureManager::getSubTexture(const std::string &mainName, int subNum){
      MainTexture *tmpTxt = getMainTexture(mainName);
      if(tmpTxt && (unsigned)subNum < tmpTxt->subTextures.size() && subNum >= 0){
         std::map<std::string, SubTexture>::iterator node = tmpTxt->subTextures.begin();
         for(;subNum > 0 && node != tmpTxt->subTextures.end();subNum--){
            node++;
         }
         if(node!=tmpTxt->subTextures.end()){
            return &(node->second);
         }
         return NULL;
      }
      return NULL;
   }
   bool TextureManager::deleteMainTexture(const std::string &mainName){
      if(mainTextures.find(mainName) != mainTextures.end()){
         mainTextures.erase(mainName);
         return 1;
      }else{
         return 0;
      }
   }
   bool TextureManager::deleteMainTexture(int num){
      std::map<std::string, MainTexture>::iterator node = mainTextures.begin();
      for(;num > 0 && node != mainTextures.end();num--){
         node++;
      }
      if(node!=mainTextures.end()){
         mainTextures.erase(node);
         return 1;
      }
      return 0;
   }

   SubTexture *TextureManager::getSubTexture(MainTexture &mainText, int subNum){
      std::map<std::string, SubTexture>::iterator node = mainText.subTextures.begin();
      for(;subNum>0 && node != mainText.subTextures.end();subNum--){
         node++;
      }
      if(node!=mainText.subTextures.end()){
         return &(node->second);
      }
      return NULL;
   }
   SubTexture *TextureManager::getSubTexture(MainTexture &mainText, const std::string &subName){
      if(mainText.subTextures.find(subName) != mainText.subTextures.end()){
         return &(mainText.subTextures[subName]);
      }
      return NULL;
   }
}
