#include "exampleCode.h"

void CreateTextScene(M2Rend::WorldScene &mainScene, M2Rend::TextLibrary &textLibrary){
   //load the font
   textLibrary.loadFont("annabel", 20, "Assets/Fonts/AnnabelScript.ttf");

   //Add basic instructions
   mainScene.addScene(textLibrary.composeScene("annabel", "Press Space to Jump"), "instruction");

   //Add background text
   M2Rend::Scene *text = mainScene.addScene(textLibrary.composeScene("annabel", "A quick brown fox jumped over the lazy dog"), "text");

   //Prepare text to rotate around its center.
   text->translate(150, 250, 0);
   M2Rend::BoxAABB textSize = text->getAABB();
   text->setRotateOrigin((textSize.maxPoint.x - textSize.minPoint.x) / 2, (textSize.maxPoint.y - textSize.minPoint.y) / 2, 0.0);
}

void ManipulateText(M2Rend::WorldScene &mainScene){
   static float red = 1.0f, green = 0.0f, blue = 0.5f;
   static float redInc = -.005f, greenInc = .010f, blueInc = .001f;
   static int direction = 1, calls = 0, callsNeeded = 300;
   calls++;
   if(calls > callsNeeded){calls = 0; callsNeeded = 600; direction*=-1;}
   //cycle the colors
   red+=redInc; green+=greenInc; blue+=blueInc;
   if(red < 0.0){red = 1.0;}
   if(blue > 1.0){blue = 0.0;}
   if(green > 1.0){green = 0.0;}

   mainScene.getScene("text")->incrementRotate(0.0, 0.0, .05 * direction);
   mainScene.getScene("text")->setColor(red, blue, green, 1.0);
}

M2Rend::DrawRectangle* GetDogShape(M2Rend::WorldScene &mainScene){
   return mainScene.getScene("DogFox")->getChild("dog")->getThis<M2Rend::DrawRectangle>();
}

M2Rend::DrawRectangle* GetFoxShape(M2Rend::WorldScene &mainScene){
   return mainScene.getScene("DogFox")->getChild("fox")->getThis<M2Rend::DrawRectangle>();
}

void LoadTexturesAndAnimations(M2Rend::TextureManager &textures, M2Rend::FrameSwapperRegister &animationLibrary){
   M2Rend::FrameSwapperDefinition frameDef;
   //Load tiled image
   textures.loadTexture("DogAndFox", "Assets/Images/dogfox.png");

   //Ground
   textures.addSubTexture("DogAndFox", "Ground", 128*2, 128*3, 256, 128);

   //Dog Standing
   for(int frame = 0;frame < 4;frame++){
      textures.addSubTexture("DogAndFox", "Dog_"+boost::lexical_cast<std::string>(frame), 128*frame, 0, 128, 128);
      frameDef.addFrame("DogAndFox", "Dog_"+boost::lexical_cast<std::string>(frame));
   }
   animationLibrary.newDefinition(frameDef, "DogStand");
   frameDef.clear();

   //Fox Standing
   for(int frame = 0;frame < 3;frame++){
      textures.addSubTexture("DogAndFox", "FoxStand_"+boost::lexical_cast<std::string>(frame), 128*frame, 128, 128, 128);
      frameDef.addFrame("DogAndFox", "FoxStand_"+boost::lexical_cast<std::string>(frame));
   }
   frameDef.addFrame("DogAndFox", "FoxStand_1");
   animationLibrary.newDefinition(frameDef, "FoxStand");
   frameDef.clear();

   //Fox Bouncing
   for(int frame = 0;frame < 3;frame++){
      textures.addSubTexture("DogAndFox", "FoxBounce_"+boost::lexical_cast<std::string>(frame), 128*frame, 128*2, 128, 128);
      frameDef.addFrame("DogAndFox", "FoxBounce_"+boost::lexical_cast<std::string>(frame));
   }
   frameDef.addFrame("DogAndFox", "FoxBounce_1");
   animationLibrary.newDefinition(frameDef, "FoxBounce");
   frameDef.clear();

   //Sun and moon
   textures.addSubTexture("DogAndFox", "Sun", 0, 128*3, 128, 128);
   textures.addSubTexture("DogAndFox", "Moon", 128, 128*3, 128, 128);
}

void CreateDogFoxScene(M2Rend::WorldScene &mainScene, M2Rend::TextureManager &textures){
   M2Rend::Scene *dogFoxScene = mainScene.makeNewScene("DogFox");

   //These values are hard coded, I could have substituted the numbers for named constants
   //but the scope of this example is incredibly limited.
   M2Rend::DrawShape *dog = new M2Rend::DrawRectangle;
   dog->getThis<M2Rend::DrawRectangle>()->setTwoCorners(350, 250, 2, 450, 350, 2);
   dogFoxScene->add(dog, "dog");

   M2Rend::DrawShape *fox = new M2Rend::DrawRectangle;
   fox->getThis<M2Rend::DrawRectangle>()->setTwoCorners(290, 250, 3, 390, 350, 3);
   dogFoxScene->add(fox, "fox");

   M2Rend::DrawShape *ground = new M2Rend::DrawRectangle;
   ground->getThis<M2Rend::DrawRectangle>()->setTwoCorners(300, 325, 1, 500, 425, 1);
   ground->getDepth();
   M2Rend::AssignTextureToRectangle(*ground->getThis<M2Rend::DrawRectangle>(), textures.getSubTexture("DogAndFox", "Ground"));
   dogFoxScene->add(ground, "ground");

   M2Rend::Scene *sunAndMoon = mainScene.makeNewScene("SunAndMoon");
   
   M2Rend::DrawShape *sun = new M2Rend::DrawRectangle;
   sun->getThis<M2Rend::DrawRectangle>()->setTwoCorners(350, 0, 1, 450, 100, 1);
   M2Rend::AssignTextureToRectangle(*sun->getThis<M2Rend::DrawRectangle>(), textures.getSubTexture("DogAndFox", "Sun"));
   sunAndMoon->add(sun, "sun");

   M2Rend::DrawShape *moon = new M2Rend::DrawRectangle;
   moon->getThis<M2Rend::DrawRectangle>()->setTwoCorners(350, 500, 1, 450, 600, 1);
   M2Rend::AssignTextureToRectangle(*moon->getThis<M2Rend::DrawRectangle>(), textures.getSubTexture("DogAndFox", "Moon"));
   sunAndMoon->add(moon, "moon");

   sunAndMoon->setRotateOrigin(400, 300, 1);
}

void UpdateSky(M2Rend::WorldScene &mainScene){
   mainScene.getScene("SunAndMoon")->incrementRotate(0.0, 0.0, -0.2);
   M2Rend::Point position = mainScene.getScene("SunAndMoon")->getRotation();
   double distance = abs(M2Util::boundBetween(position.z, 0.0, 360.0) - 180.0);
   float brightness = distance / 180.0;
   if(brightness < .1){brightness = .1;}

   mainScene.getRenderer()->setBackgroundColor(M2Rend::Color(0.25f*brightness, 0.45f*brightness, 0.65f*brightness));
}

void UpdateAnimation(M2Rend::WorldScene &mainScene, M2Rend::TextureManager &textures, M2Rend::DrawRectangle &shape, M2Rend::FrameSwapper &animation, bool flipAnimation){
   std::string mainTexture, subTexture;
   animation.getCurrentFrame(mainTexture, subTexture);
   AssignTextureToRectangle(shape, textures.getSubTexture(mainTexture, subTexture), flipAnimation);
}

FoxJump::FoxJump( M2Rend::FrameSwapperRegister *frameReg, M2Rend::FrameSwapper *foxAnim, M2Rend::DrawRectangle *foxShape ) :
   foxBox(foxShape),
   foxSwapper(foxAnim),
   frameRegister(frameReg),
   inJump(false),
   movedX(0),
   directionX(1),
   distanceX(100),distanceY(100),
   jumpCompletionTime(1000){}

void FoxJump::initiateJump(){
   if(!inJump){
      inJump = true;
      movedY = 0;
      directionY = -1;
      foxSwapper->setFrameList(frameRegister->getDefinition("FoxBounce"));
      timer.start();
      timer.delta("jumped");
   }
}

void FoxJump::updateJump(){
   if(inJump){
      double timeElapsed = timer.delta("jumped");
      movedX += directionX * (distanceX * (timeElapsed / jumpCompletionTime));
      movedY += directionY * (distanceY * (timeElapsed / (jumpCompletionTime / 2.0)));
      if(movedY < -distanceY){directionY = 1; movedY = -distanceY;}
      if(movedX > distanceX || movedX < 0){
         movedY = 0;
         if(movedX > distanceX){
            movedX = distanceX;
         }else if(movedX < 0){
            movedX = 0;
         }
         directionX*=-1;
         inJump = false;
         foxSwapper->setFrameList(frameRegister->getDefinition("FoxStand"));
      }
      foxBox->placeAt(movedX, movedY, 3);
   }
}

bool FoxJump::isFlipped(){
   return directionX == 1;
}