/*/
Grab a drink and something to nibble on and enjoy reading through this live
example of my 2d engine.  The project requires Visual Studio 2010 to compile
as it makes use of Lambda functions and other C++0x features where it 
simplifies the code base.  Primarily in use of the std::for_each function.

If you cannot compile this project I have included the binary (compiled in
Windows XP, 32 bit) if there is a problem with the binary let me know.
/*/

#include "exampleCode.h"

bool acceptInput(SDL_Event &event, FoxJump &foxJump);
void quit(void);

int main(int argc, char *argv[]){
   srand ((unsigned int)time(0));

   //RENDERER SETUP:::::::::::::::::::::::::::::::::
   M2Rend::Draw2D renderer;
   renderer.setBackgroundColor(M2Rend::Color(0.25f, 0.45f, 0.65f));
   M2Rend::Point worldSize(800, 600, 0), windowSize(800, 600, 0);

   renderer.useFullScreen(false);
   if(!renderer.initialize((int)windowSize.x, (int)windowSize.y, (int)worldSize.x, (int)worldSize.y)){
      exit(0);
   }
   atexit(quit);


   //WORLD SETUP::::::::::::::::::::::::::::::
   //Set up managing objects
   M2Rend::TextureManager textures;
   M2Rend::FrameSwapperRegister animationLibrary;
   LoadTexturesAndAnimations(textures, animationLibrary);

   M2Rend::TextLibrary textLibrary(&renderer);
   M2Rend::WorldScene mainScene(&renderer);

   CreateTextScene(mainScene, textLibrary);
   CreateDogFoxScene(mainScene, textures);

   //Initialize the individual animation objects for the dog and fox.
   M2Rend::FrameSwapper dogAnimation, foxAnimation;
   dogAnimation.setFrameList(animationLibrary.getDefinition("DogStand"));
   dogAnimation.start();

   foxAnimation.setFrameList(animationLibrary.getDefinition("FoxStand"));
   foxAnimation.start();

   //Create the fox jump state object
   FoxJump foxJump(&animationLibrary, &foxAnimation, GetFoxShape(mainScene));

   //AUDIO SETUP::::::::::::::::::::::::::::::
   AudioPlayer *player = AudioPlayer::instance();
   player->initAudio();
   player->loadMusic("Assets/Audio/lifefortwo.mp3", "RepeatSong1");
   player->loadMusic("Assets/Audio/countrysides.mp3", "RepeatSong2");

   AudioPlayList MusicPlayList;
   player->setMusicPlayList(&MusicPlayList);
   MusicPlayList.setPlayListType(MUSIC_PLAYLIST);
   MusicPlayList.addSoundBack("RepeatSong1");
   MusicPlayList.addSoundBack("RepeatSong2");
   MusicPlayList.resetPlayHead();
   MusicPlayList.loopSounds(true);
   MusicPlayList.continuousPlay(true);


   MusicPlayList.beginPlaying();

   //GAME LOOP::::::::::::::::::::::::::::::::::::
   SDL_Event event;
   bool done = false;
   while(!done){
      //Handle window closing/SDL quitting, and the jump button
      done = acceptInput(event, foxJump);
      
      //Spin the text and alter its color and spin the sky
      ManipulateText(mainScene);
      UpdateSky(mainScene);

      //Update the fox position and animation based on the jump state
      foxJump.updateJump();

      //Update the animation on the fox and dog
      UpdateAnimation(mainScene, textures, *GetDogShape(mainScene), dogAnimation);
      UpdateAnimation(mainScene, textures, *GetFoxShape(mainScene), foxAnimation, foxJump.isFlipped());

      //Update the screen
      renderer.clearScreen();
      mainScene.draw();
      renderer.updateScreen();

      //Delay a moment to avoid hogging the CPU
      SDL_Delay(5);
   }
   return 0;
}

bool acceptInput(SDL_Event &event, FoxJump &foxJump){
   while(SDL_PollEvent(&event)){
      switch(event.type){
      case SDL_QUIT:
         return true;
         break;
      case SDL_KEYDOWN:
         switch(event.key.keysym.sym){
         case SDLK_ESCAPE:
            return true;
            break;
         case SDLK_SPACE:
            foxJump.initiateJump();
            break;
         }
         break;
      }
   }
   return false;
}

void quit(void){
   SDL_Quit();
   TTF_Quit();
}
